/*
   DVR, a Digital video Recorder for Linux.
   
   Copyright (C) 2001-2007 Pierre Hebert <pierrox@pierrox.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this program; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

 */

#include <qcombobox.h>
#include <qtabwidget.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qspinbox.h>
#include <qlistbox.h>
#include <qcheckbox.h>

#include "tab_general.h"
#include "tab_monitoring.h"
#include "tab_video_config.h"
#include "tab_audio_config.h"
#include "tab_codec_config.h"
#include "device_selection.h"
#include "dvr_control.h"

DVRControl::DVRControl() : UIDVRControl() {
  QString title;
  title.sprintf(tr("DVR : a Digital Video Recorder for Linux (version %s )"), DVR_VERSION);
  setCaption(title);

  bool ok=false;
  DeviceSelection device_selection;
  do {
    if(device_selection.exec()==QDialog::Accepted) {
      if(device_selection.manual_selection) {
        device=device_selection.qle_device->text();
        tab_video_config->qlb_video_device->setText(device);
      } else {
        DeviceDesc selected;
        DeviceList::iterator it;
        QString name=device_selection.qcb_device->currentText();
        for(it=device_selection.devices_list.begin(); it!=device_selection.devices_list.end(); ++it) {
          if((*it).first==name) {
            device=(*it).second;
            tab_video_config->qlb_video_device->setText(device+" ("+name+")");
            break;
          }
        }
      }
      
      ok=true;
    } else {
      int answer=QMessageBox::warning(this, tr("Video device selection"), 
                                            tr("You did not choose a video device and so DVR won't be able to record any video.\nDo you want to retry ?"),
                                            QMessageBox::Yes, QMessageBox::No);
      ok=(answer==QMessageBox::No);
    }
  } while(!ok);

  if(device=="") {
    qpb_start->setEnabled(false);
    qpb_stop->setEnabled(false);
  }

  tab_video_config->tab_codec_config->findCodecs("avimux", (const char*[]){"video/x-raw-rgb", "video/x-raw-yuv", NULL} , "Codec/Encoder/Video");
  tab_audio_config->tab_codec_config->findCodecs("avimux", (const char*[]){"audio/x-raw-int", NULL} , "Codec/Encoder/Audio");
}

DVRControl::~DVRControl() {
}

void DVRControl::startRecording() {
  int width, height;
  int framerate;
  
  sscanf(tab_video_config->qcb_predefined_size->currentText(), "%dx%d", &width, &height);
  framerate=tab_video_config->qsb_video_framerate->value();
  
  dvr.setImageSize(width, height);
  dvr.setFrameRate(framerate);
  dvr.setDeviceFileName(device);
  dvr.setVideoFileName(tab_general->fileName());
  dvr.setVideoCodec(tab_video_config->tab_codec_config->selectedCodec());
  if(tab_audio_config->qcb_sound_recording_enabled->isChecked()) {
    dvr.setAudioCodec(tab_audio_config->tab_codec_config->selectedCodec());
  } else {
    dvr.setAudioCodec(NULL);
  }
  
  if(dvr.start()) {
    qpb_start->setEnabled(false);
    qpb_stop->setEnabled(true);
    tab_monitoring->start(tab_general->fileName());
    tw_configuration->setCurrentPage(3);
    qlb_status->setText("Recording...");
  } else {
      QMessageBox::critical(this, tr("Error"), 
                                  tr("Sorry, the capture failed (more messages on stdout/stderr)"),
                                  QMessageBox::Ok, QMessageBox::NoButton);
  }
}

void DVRControl::stopRecording() {
  dvr.stop();
  tab_monitoring->stop();
  qpb_start->setEnabled(true);
  qpb_stop->setEnabled(false);
  qlb_status->setText("Ready.");
}
