/*
   DVR, a Digital video Recorder for Linux.
   
   Copyright (C) 2001-2007 Pierre Hebert <pierrox@pierrox.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this program; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

 */

#include <qlistbox.h>
#include <qlistview.h>
#include <gst/gst.h>


#include "property_editor.h"
#include "qlistbox_codec.h"
#include "qlistview_codec_property.h"
#include "tab_codec_config.h"

TabCodecConfig::TabCodecConfig(QWidget *parent, const char *name) : 
  UITabCodecConfig(parent, name) 
{
}

TabCodecConfig::~TabCodecConfig() {}

void TabCodecConfig::findCodecs(const char *muxer, const char **types, const char *klass) {
  GstElementFactory *factory;
  factory=gst_element_factory_find(muxer);
  if(factory) {
    GstElementFactory *factory2;
    factory2=GST_ELEMENT_FACTORY(gst_plugin_feature_load(GST_PLUGIN_FEATURE(factory)));
    if(factory2) {
      GstElement *element;
      element=gst_element_factory_create(factory2, NULL);
      if(element) {
        const GList *pads;
        GstStaticPadTemplate *padtemplate;
        
        pads=factory->staticpadtemplates;
        while(pads) {
          padtemplate=(GstStaticPadTemplate *)(pads->data);
          pads=g_list_next(pads);

          if(padtemplate->direction==GST_PAD_SINK) {
            if(padtemplate->static_caps.string) {
              const GstCaps*caps=gst_static_caps_get(&padtemplate->static_caps);
              if(caps) {
                guint i;
                for(i=0; i<gst_caps_get_size(caps); i++) {
                  GstStructure *structure=gst_caps_get_structure(caps, i);
                  findCodecsForCap(gst_structure_get_name(structure), types, klass);
                }
              } else {
                fprintf(stderr, "error : gst_static_caps_get\n");
              }
            }
          }
        }
        
        gst_object_unref(element);
      } else {
        fprintf(stderr, "error : gst_element_factory_create\n");
      }
      
    } else {
      fprintf(stderr, "error : GST_ELEMENT_FACTORY (gst_plugin_feature_load (GST_PLUGIN_FEATURE (factory)))\n");
    }

    gst_object_unref (factory);
  } else {
    fprintf(stderr, "error : gst_element_factory_find\n");
  }

  qlb_codec_list->setSelected(0, true);
}


void TabCodecConfig::findCodecsForCap(const char *cap_name, const char **types, const char *klass) {
  GList *plugins, *orig_plugins;
  orig_plugins=plugins=gst_default_registry_get_plugin_list();
  while (plugins) {
    GList *features, *orig_features;
    GstPlugin *plugin;
    plugin=(GstPlugin *)(plugins->data);
    plugins=g_list_next (plugins);

    orig_features=features=gst_registry_get_feature_list_by_plugin (gst_registry_get_default (), plugin->desc.name);
    while (features) {
      GstPluginFeature *feature;

      feature=GST_PLUGIN_FEATURE(features->data);

      if(GST_IS_ELEMENT_FACTORY(feature)) {
        GstElementFactory *factory;

        factory=GST_ELEMENT_FACTORY (feature);
        if(!strcmp(factory->details.klass, klass)) {
          GstElement *element;
          element=gst_element_factory_create(factory, NULL);
          if(element) {
            bool src_ok, sink_ok;
            const GList *pads;
            GstStaticPadTemplate *padtemplate;
            
            src_ok=false;
            sink_ok=false;
            
            pads=factory->staticpadtemplates;
            while(pads) {
              padtemplate=(GstStaticPadTemplate *)(pads->data);
              pads=g_list_next (pads);

              if(padtemplate->direction==GST_PAD_SRC) {
                if(padtemplate->static_caps.string) {
                  const GstCaps *caps=gst_static_caps_get(&padtemplate->static_caps);
                  if(caps) {
                    guint i;
                    for(i=0; i<gst_caps_get_size(caps); i++) {
                      GstStructure *structure=gst_caps_get_structure(caps, i);
                      if(!strcmp(cap_name, gst_structure_get_name(structure))) {
                        src_ok=true;
                      }
                    }
                  } else {
                    fprintf(stderr, "error : gst_static_caps_get\n");
                  }
                }
              } else if(padtemplate->direction==GST_PAD_SINK) {
                if(padtemplate->static_caps.string) {
                  const GstCaps *caps=gst_static_caps_get(&padtemplate->static_caps);
                  if(caps) {
                    guint i;
                    for(i=0; i<gst_caps_get_size(caps); i++) {
                      GstStructure *structure=gst_caps_get_structure (caps, i);
                      const char **type=types;
                      while(*type) {
                        if(!strcmp(*type, gst_structure_get_name (structure))) {
                          sink_ok=true;
                          break;
                        }
                      
                        type++;
                      }
                    }
                  } else {
                    fprintf(stderr, "error : gst_static_caps_get\n");
                  }
                }
              }

              if(src_ok && sink_ok) {
                new QListBoxCodec(qlb_codec_list, factory->details.longname, GST_PLUGIN_FEATURE_NAME(factory));
              }
            }
            
            gst_object_unref(element);
          } else {
            fprintf(stderr, "error : gst_element_factory_create\n");
          }
        }
      }

      features=g_list_next(features);
    }
    gst_plugin_feature_list_free(orig_features);
  }
  gst_plugin_list_free(orig_plugins);
}

void TabCodecConfig::codecSelected(QListBoxItem *item) {
  QListBoxCodec *codec=(QListBoxCodec*)item;
  
  GstElement *element=codec->element;
  qlv_property_list->clear();

  GParamSpec **property_specs;
  guint num_properties, i;

  property_specs=g_object_class_list_properties(G_OBJECT_GET_CLASS(element), &num_properties);
  for(i=0; i<num_properties; i++) {
    GParamSpec *param=property_specs[i];
    if(strcmp(param->name, "name") && (param->flags & (G_PARAM_READABLE | G_PARAM_WRITABLE))) {
      new QListViewCodecProperty(qlv_property_list, element, param->name, g_param_spec_get_blurb(param), propertyValue(element, param->name));
    }
  }
}

GstElement *TabCodecConfig::selectedCodec() {
  QListBoxCodec *codec=(QListBoxCodec*)qlb_codec_list->selectedItem();
  return codec->element;
}

void TabCodecConfig::editProperty(QListViewItem *property) {
  QListViewCodecProperty *prop=(QListViewCodecProperty*)property;
  
  PropertyEditor prop_edit(this, prop->element, prop->name);

  if(prop_edit.exec()==QDialog::Accepted) {
    property->setText(1, propertyValue(prop->element, prop->name));
  }
}

QString TabCodecConfig::propertyValue(GstElement *element, const char *property_name) {
  QString ret;
  GParamSpec *param=g_object_class_find_property(G_OBJECT_GET_CLASS(element), property_name);

  GValue value={ 0, 0 };
  g_value_init(&value, param->value_type);
  g_object_get_property(G_OBJECT(element), param->name, &value);
  
  switch(G_VALUE_TYPE(&value)) {
    case G_TYPE_STRING :
      const char *s=g_value_get_string(&value);
      if(s==NULL) s="<null>";
      ret=s;
      break;
      
    case G_TYPE_BOOLEAN :
      if(g_value_get_boolean(&value)) {
        ret=tr("yes");
      } else {
        ret=tr("no");
      }
      break;
      
    case G_TYPE_ULONG : {
      ret.sprintf("%lu", g_value_get_ulong(&value));
      break;  
    }
    
    case G_TYPE_LONG : {
      ret.sprintf("%ld", g_value_get_long(&value));
      break;  
    }
    
    case G_TYPE_UINT : {
      ret.sprintf("%u", g_value_get_uint(&value));
      break;
    }

    case G_TYPE_INT : {
      ret.sprintf("%d", g_value_get_int(&value));
      break;
    }

    case G_TYPE_UINT64 : {
      ret.sprintf("%"G_GUINT64_FORMAT, g_value_get_uint64(&value));
      break;
    }

    case G_TYPE_INT64 : {
      ret.sprintf("%"G_GINT64_FORMAT, g_value_get_int64(&value));
      break;
    }

    case G_TYPE_FLOAT : {
      ret.sprintf("%g", g_value_get_float(&value));
      break;
    }

    case G_TYPE_DOUBLE : {
      ret.sprintf("%g", g_value_get_double(&value));
      break;
    }
      
    default:
      if(G_IS_PARAM_SPEC_ENUM(param)) {
        guint j;
        gint val;
        GEnumValue *values;
        values=G_ENUM_CLASS(g_type_class_ref(param->value_type))->values;
        val=g_value_get_enum(&value);
        j=0;
        while(values[j].value_name) {
          if(values[j].value==val) {
            ret=values[j].value_name;
            break;
          }
          j++;
        }
      } else {
        ret="<unsupported type !>";
      }
      break;
  }

  return ret;
}
